from abc import ABC, abstractmethod
from enum import Enum
from typing import Dict, List


class OperatingSystem(Enum):
    Linux = 'Linux'
    Windows = 'Windows'
    MacOS = 'Darwin'


class PackageManager(ABC):

    @classmethod
    def operating_system(cls) -> OperatingSystem:
        import platform
        if platform.system() == 'Linux':
            return OperatingSystem.Linux
        elif platform.system() == 'Windows':
            return OperatingSystem.Windows
        elif platform.system() == 'Darwin':
            return OperatingSystem.MacOS

    @classmethod
    @abstractmethod
    def command(cls) -> List[str]:
        pass

    @classmethod
    @abstractmethod
    def canonical_name(cls) -> str:
        pass

    @classmethod
    @abstractmethod
    def setup(cls) -> bool:
        pass

    @classmethod
    @abstractmethod
    def install(cls, packages: Dict[str, List[str]]) -> bool:
        pass

    @classmethod
    @abstractmethod
    def update(cls) -> bool:
        pass

    @classmethod
    @abstractmethod
    def upgrade(cls) -> bool:
        pass

    @classmethod
    @abstractmethod
    def remove(cls, packages: Dict[str, List[str]]) -> bool:
        pass

    @classmethod
    @abstractmethod
    def check(cls) -> bool:
        return False

    @classmethod
    @abstractmethod
    def clean(cls) -> bool:
        pass
