from shutil import which
from subprocess import call
from typing import List, Dict

from skip.managers.manager import PackageManager, OperatingSystem


class Yum(PackageManager):
    @classmethod
    def remove(cls, packages: Dict[str, List[str]]) -> bool:
        pass

    @classmethod
    def command(cls) -> List[str]:
        return "yum".strip().split()

    @classmethod
    def canonical_name(cls) -> str:
        return "yum"

    @classmethod
    def upgrade(cls) -> bool:
        cmd_lst = ["sudo", "-E"]
        cmd_lst.extend(cls.command())
        cmd_lst.extend(["upgrade", "-y"])

        return call(cmd_lst, stdin=True) == 0

    @classmethod
    def clean(cls) -> bool:
        cmd_lst = ["sudo", "-E"]
        cmd_lst.extend(cls.command())
        cmd_lst.extend(["autoremove", "-y"])

        return call(cmd_lst, stdin=True) == 0

    @classmethod
    def setup(cls):
        pass

    @classmethod
    def install(cls, packages: Dict[str, List[str]]) -> bool:
        pass

    @classmethod
    def update(cls) -> bool:
        cmd_lst = ["sudo", "-E"]
        cmd_lst.extend(cls.command())
        cmd_lst.extend(["updateinfo", "-y"])

        return call(cmd_lst, stdin=True) == 0

    @classmethod
    def check(cls) -> bool:
        return cls.operating_system() in {OperatingSystem.Linux} and \
               which(cls.command()[0]) is not None
